@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2010 TSUTSUMI Kikuo.
+/* Copyright (C) 2003, 2010, 2013 TSUTSUMI Kikuo.
    This file is part of the CCUnit Library.
 
    The CCUnit Library is free software; you can redistribute it and/or
@@ -17,7 +17,7 @@
    Place - Suite 330, Boston, MA 02111-1307, USA.  
 */
 /*
- * $Id: CCUnitReadSuite.c,v 1.16 2010/08/16 05:07:00 tsutsumi Exp $
+ * $Id$
  */
 
 /** @file
@@ -28,6 +28,8 @@
 #include <ctype.h>
 #include <errno.h>
 
+static void readSuite (_CCUnitTestSuiteDef* suite);
+
 /**
  * @addtogroup CCUnitReadSuite
  * @{
@@ -143,14 +145,14 @@ static inline _CCUnitTestDef* addTestSuiteDef (_CCUnitTestSuiteDef* suite,
 }
 
 /**
- * add test fixture to test suite.
+ * add test case to test suite.
  *
  * @param suite test suite to add.
- * @param test test fixture.
+ * @param test test case.
  * @return added test.
  */
-static inline _CCUnitTestDef* addTestFixtureDef (_CCUnitTestSuiteDef* suite,
-						 _CCUnitTestFixtureDef* test)
+static inline _CCUnitTestDef* addTestCaseDef (_CCUnitTestSuiteDef* suite,
+						 _CCUnitTestCaseDef* test)
 {
   const char* name;
   if (!suite || !test)
@@ -158,18 +160,18 @@ static inline _CCUnitTestDef* addTestFixtureDef (_CCUnitTestSuiteDef* suite,
   name = test->testdef.name;
   if (!name)
     name = "";
-  ccunit_log ("add test fixture: %s", name);
+  ccunit_log ("add test case: %s", name);
   return addTestDef (suite, &test->testdef);
 }
 
 /**
- * create new test case.
+ * create new test func.
  *
  * @param scope scope.
- * @param type return type of case.
- * @param name case name.
+ * @param type return type of func.
+ * @param name func name.
  * @param desc description.
- * @return new test case def.
+ * @return new test func def.
  */
 static _CCUnitFuncDef* newFuncDef (const char* scope,
 				   const char* type,
@@ -203,32 +205,30 @@ static void deleteFuncDef (_CCUnitFuncDef* func)
 }
 
 /**
- * destroy test fixture def.
- * @param fixture test fixture def to destroy.
+ * destroy test case def.
+ * @param testCase test case def to destroy.
  */
-static void destroyTestFixtureDef (_CCUnitTestFixtureDef* fixture)
+static void destroyTestCaseDef (_CCUnitTestCaseDef* testCase)
 {
-  ccunit_deleteList (&fixture->testCases, (void(*)(void*))deleteFuncDef);
-  deleteFuncDef (fixture->setUp);
-  deleteFuncDef (fixture->tearDown);
-  destroyTestDef (&fixture->testdef);
+  ccunit_deleteList (&testCase->testFuncs, (void(*)(void*))deleteFuncDef);
+  destroyTestDef (&testCase->testdef);
 }
 
 /**
- * create new test fixture def.
+ * create new test case def.
  *
- * @param name test fixture name.
+ * @param name test case name.
  */
-static _CCUnitTestFixtureDef* newTestFixtureDef (const char* name)
+static _CCUnitTestCaseDef* newTestCaseDef (const char* name)
 {
-  _CCUnitTestFixtureDef* fixture = calloc (1, sizeof (*fixture));
-  ccunit_log ("create new test fixture: %s", name);
-  if (!fixture)
+  _CCUnitTestCaseDef* testCase = calloc (1, sizeof (*testCase));
+  ccunit_log ("create new test case: %s", name);
+  if (!testCase)
     return NULL;
-  initTestDef (&fixture->testdef, ccunitTypeFixture, name);
-  fixture->testdef.dtor = (void(*)(_CCUnitTestDef*))destroyTestFixtureDef;
-  ccunit_initList (&fixture->testCases);
-  return fixture;
+  initTestDef (&testCase->testdef, ccunitTypeTestCase, name);
+  testCase->testdef.dtor = (void(*)(_CCUnitTestDef*))destroyTestCaseDef;
+  ccunit_initList (&testCase->testFuncs);
+  return testCase;
 }
 
 /**
@@ -257,7 +257,7 @@ static struct _CCUnitLine line;
 
 /**
  * Get one line from stream.
- * This case copies a read line on the global variable <code>line</code>.
+ * This func copies a read line on the global variable <code>line</code>.
  *
  * @return When reading succeeds, value except for the zero is
  * returned. When an error occurs, a zero is returned.
@@ -358,7 +358,7 @@ static char* readDocCommentContents ()
     {
       ccunit_dbg ("read from:%lu: \"%s\"", line.lno, start);
       /* skip white spaces */
-      for (; *start && isspace (*start); start ++)
+      for (; *start && isspace ((int)*start); start ++)
 	;
       /* skip leading block comment '*'<WSP>... */
       if (*start != '*')
@@ -366,7 +366,7 @@ static char* readDocCommentContents ()
       else if (start[1] == '/')			/* eoc */
 	;
       else					/* skip white spaces */
-	for (start ++; *start && isspace (*start); start ++)
+	for (start ++; *start && isspace ((int)*start); start ++)
 	  ;
       /* skip doxygen command line */
       if (*start == '@')
@@ -391,7 +391,7 @@ static char* readDocCommentContents ()
 	  }
       /* trim trailing white space */
       for (end --; start < end; end --)
-	if (!isspace (*end))
+	if (!isspace ((int)*end))
 	  {
 	    end ++;
 	    break;
@@ -429,8 +429,8 @@ static char* readDocCommentContents ()
  */
 static char* readDocComment ()
 {
-  const char* cmnt = "/**";
-  if (strncmp (line.str, cmnt, strlen(cmnt)) != 0) /* not a comment */
+  static const char cmnt[] = "/**";
+  if (strncmp (line.str, cmnt, sizeof (cmnt) - 1) != 0) /* not a comment */
     ;
   else if (line.str[3] == '*' || line.str[3] == '/') /* not doc */
     ;
@@ -443,7 +443,7 @@ static char* readDocComment ()
 }
 
 static const char* testTypeStr[] = {
-  "fixture", "suite"
+  "case", "suite"
 };
 
 /**
@@ -466,7 +466,7 @@ static const char* getTestName (CCUnitTestType_t type, const char* str)
   if (strncasecmp (str, prefix, prefixLen) == 0)
     {
       for (name = str + prefixLen; *name; name ++)
-	if (!isspace (*name))
+	if (!isspace ((int)*name))
 	  break;
       if (!*name)
 	{
@@ -500,15 +500,15 @@ getEndOfTest (CCUnitTestType_t type, const char* str)
   if (strncasecmp (str, prefix, prefixLen) == 0)
     {
       name = str + prefixLen;
-      if (*name && !isspace (*name) && !ispunct (*name))
+      if (*name && !isspace ((int)*name) && !ispunct ((int)*name))
 	{
 	  name = NULL;
-	  ccunit_dbg ("not a end of test %s: %s", testType, str);
+	  ccunit_dbg ("not an end of test %s: %s", testType, str);
 	}
       else
 	{
 	  for (; *name; name ++)
-	    if (!isspace (*name))
+	    if (!isspace ((int)*name))
 	      break;
 	  if (!*name)
 	    ;
@@ -517,35 +517,35 @@ getEndOfTest (CCUnitTestType_t type, const char* str)
 	}
     }
   else
-    ccunit_dbg ("not a end of test %s: %s", testType, str);
+    ccunit_dbg ("not an end of test %s: %s", testType, str);
   return name;
 }
 
 /**
- * read test casedef.
+ * read test funcdef.
  *
  * @param type required type string.
- * @param prefix required case name prefix.
+ * @param prefix required func name prefix.
  * @param desc description.
  * @return funcdef object.
  */
-static _CCUnitFuncDef* readTestCase (const char* type,
+static _CCUnitFuncDef* readTestFunc (const char* type,
 				    const char* prefix,
 				    const char* desc)
 {
   const char* scope = "static";
   char* typ;
   char* name;
-  ccunit_dbg ("read case: %s %s... from '%s'", type, prefix, line.str);
+  ccunit_dbg ("read func: %s %s... from '%s'", type, prefix, line.str);
   for (typ = line.str; *typ; typ ++)
-    if (!isspace (*typ))
+    if (!isspace ((int)*typ))
       break;
   if (strncmp (typ, scope, strlen (scope)) != 0)
     scope = "extern";
   else
     {
       typ += strlen (scope);
-      if (*typ && !isspace (*typ))
+      if (*typ && !isspace ((int)*typ))
 	{
 	  ccunit_dbg ("type mismatch: %s %s", type, typ);
 	  return NULL;
@@ -553,7 +553,7 @@ static _CCUnitFuncDef* readTestCase (const char* type,
       for (;;)
 	{
 	  for (; *typ; typ ++)
-	    if (!isspace (*typ))
+	    if (!isspace ((int)*typ))
 	      break;
 	  if (*typ)
 	    break;
@@ -571,7 +571,7 @@ static _CCUnitFuncDef* readTestCase (const char* type,
       return NULL;
     }
   name = typ + strlen (type);
-  if (*name && !isspace (*name))
+  if (*name && !isspace ((int)*name))
     {
       ccunit_dbg ("type mismatch: %s %s", type, name);
       return NULL;
@@ -579,7 +579,7 @@ static _CCUnitFuncDef* readTestCase (const char* type,
   for (;;)
     {
       for (; *name; name ++)
-	if (!isspace (*name))
+	if (!isspace ((int)*name))
 	  break;
       if (*name)
 	break;
@@ -594,7 +594,7 @@ static _CCUnitFuncDef* readTestCase (const char* type,
     {
       char* tail;
       for (tail = name + 1; *tail; tail ++)
-	if (isspace (*tail) || *tail == '(')
+	if (isspace ((int)*tail) || *tail == '(')
 	  {
 	    *tail = '\0';
 	    break;
@@ -607,74 +607,80 @@ static _CCUnitFuncDef* readTestCase (const char* type,
 }
 
 /**
- * set fixture function.
- * @param fpp fixture function adderss.
- * @param fp fixture function to set.
- */
-static void setFixtureFunc (_CCUnitFuncDef** fpp, _CCUnitFuncDef* fp)
-{
-  if (!*fpp)
-    *fpp = fp;
-  else
-    {
-      ccunit_err ("%s:%lu: %s definition is conflicted... ignore.\n"
-		  "    previous definition is %s\n",
-		  line.fname, line.lno, fp->name, (*fpp)->name);
-      deleteFuncDef (fp);
-    }
-}
-
-/**
- * read test fixture function.
+ * read test case function.
  *
  * @param suite parent suite.
- * @param cname test fixture name to read.
+ * @param cname test case name to read.
  */
-static void readTestFixture (_CCUnitTestSuiteDef* suite, const char* cname)
+static void readTestCase (_CCUnitTestSuiteDef* suite, const char* cname)
 {
-  _CCUnitTestFixtureDef* fixture;
+  _CCUnitTestCaseDef* testCase;
   _CCUnitFuncDef* f = NULL;
   const char* name;
   char* doc;
   char* desc = NULL;
   if (!suite)
     return;
-  fixture = newTestFixtureDef (cname);
-  if (!fixture)
+  testCase = newTestCaseDef (cname);
+  if (!testCase)
     return;
-  addTestFixtureDef (suite, fixture);
+  addTestCaseDef (suite, testCase);
   while (readline ())
     {
-      /* setUp function def */
-      if ((f = readTestCase ("void", "setUp", desc)) != NULL)
+      /* setUpBeforeClass function def */
+      if ((f = readTestFunc ("void", "setUpBeforeClass", desc)) != NULL)
 	{
-	  setFixtureFunc (&fixture->setUp, f);
+	  ccunit_addList (&testCase->testFuncs, f);
+	  safe_free (desc);
+	}
+      /* tearDownAfterClass function def */
+      else if ((f = readTestFunc ("void", "tearDownAfterClass", desc)) != NULL)
+	{
+	  ccunit_addList (&testCase->testFuncs, f);
+	  safe_free (desc);
+	}
+      /* setUp function def */
+      else if ((f = readTestFunc ("void", "setUp", desc)) != NULL)
+	{
+	  ccunit_addList (&testCase->testFuncs, f);
 	  safe_free (desc);
 	}
       /* tearDown function def */
-      else if ((f = readTestCase ("void", "tearDown", desc)) != NULL)
+      else if ((f = readTestFunc ("void", "tearDown", desc)) != NULL)
 	{
-	  setFixtureFunc (&fixture->tearDown, f);
+	  ccunit_addList (&testCase->testFuncs, f);
 	  safe_free (desc);
 	}
-      /* if test case function def, then read as test case. */
-      else if ((f = readTestCase ("void", "test", desc)) != NULL)
+      /* if test case function def, then read as test func. */
+      else if ((f = readTestFunc ("void", "test", desc)) != NULL)
 	{
-	  ccunit_addList (&fixture->testCases, f);
+	  ccunit_addList (&testCase->testFuncs, f);
 	  safe_free (desc);
 	}
       /* if current line is javaDoc comment, then read as description. */
       else if ((doc = readDocComment ()) != NULL)
 	{
-	  if ((name = getTestName (ccunitTypeFixture, doc)) != NULL)
+	  if ((name = getTestName (ccunitTypeTestCase, doc)) != NULL)
 	    {
-	      readTestFixture (suite, name);
+	      readTestCase (suite, name);
 	      safe_free (doc);
 	    }
-	  else if ((name = getEndOfTest (ccunitTypeFixture, doc))
+	  else if ((name = getTestName (ccunitTypeSuite, doc)) != NULL)
+	    {
+	      _CCUnitTestSuiteDef* newSuite;
+	      ccunit_err ("%s:%lu: missing end test case comment for '%s'",
+			  line.fname, line.lno, testCase->testdef.name);
+	      newSuite = ccunit_newTestSuiteDef (name);
+	      if (!newSuite)
+		break;
+	      addTestSuiteDef (suite, newSuite);
+	      readSuite (newSuite);
+	      safe_free (doc);
+	    }
+	  else if ((name = getEndOfTest (ccunitTypeTestCase, doc))
 		   != NULL)
 	    {
-	      ccunit_log ("end test fixture: %s", fixture->testdef.name);
+	      ccunit_log ("end test case: %s", testCase->testdef.name);
 	      safe_free (doc);
 	      break;
 	    }
@@ -704,14 +710,14 @@ static void readSuite (_CCUnitTestSuiteDef* suite)
       /* if current line is javaDoc comment, then read as description. */
       if ((doc = readDocComment ()) != NULL)
 	{
-	  if ((name = getTestName (ccunitTypeFixture, doc)) != NULL)
+	  if ((name = getTestName (ccunitTypeTestCase, doc)) != NULL)
 	    {
-	      readTestFixture (suite, name);
+	      readTestCase (suite, name);
 	      safe_free (doc);
 	    }
-	  else if ((name = getEndOfTest (ccunitTypeFixture, doc)) != NULL)
+	  else if ((name = getEndOfTest (ccunitTypeTestCase, doc)) != NULL)
 	    {
-	      ccunit_err ("%s:%lu: invalid end test fixture comment '%s'",
+	      ccunit_err ("%s:%lu: invalid end test case comment '%s'",
 			  line.fname, line.lno, doc);
 	      safe_free (doc);
 	    }
@@ -733,11 +739,13 @@ static void readSuite (_CCUnitTestSuiteDef* suite)
 	  desc = doc;
 	  doc = NULL;
 	}
-      else if ((f = readTestCase ("void", "test", desc)) != NULL
-	       || (f = readTestCase ("void", "setUp", desc)) != NULL
-	       || (f = readTestCase ("void", "tearDown", desc)) != NULL)
+      else if ((f = readTestFunc ("void", "test", desc)) != NULL
+	       || (f = readTestFunc ("void", "setUpBeforeClass", desc)) != NULL
+	       || (f = readTestFunc ("void", "tearDownAfterClass", desc)) != NULL
+	       || (f = readTestFunc ("void", "setUp", desc)) != NULL
+	       || (f = readTestFunc ("void", "tearDown", desc)) != NULL)
 	{
-	  ccunit_err ("%s:%lu: missing test fixture start comment '%s': ignored",
+	  ccunit_err ("%s:%lu: missing test case start comment '%s': ignored",
 		      line.fname, line.lno, line.str);
 	  deleteFuncDef (f);
 	  safe_free (desc);
@@ -756,7 +764,7 @@ void ccunit_readSuite (const char* fname, _CCUnitTestSuiteDef* suite)
   if (strcmp (fname, "-") == 0) /* special file name '-' as stdin  */
     {
       line.ifp = stdin;
-      line.fname = "stdin";
+      line.fname = "(stdin)";
     }
   else
     {
